From a7d5bd47e058438478c14e99ef395c4a9fc7e45c Mon Sep 17 00:00:00 2001
From: Julien 'Lta' BALLET <contact@lta.io>
Date: Wed, 8 May 2013 21:30:03 +0200
Subject: [PATCH 2/2] Adds a I420 -> NV12 chroma_filter (c implementation)

---
 modules/video_chroma/Modules.am  |  5 +++
 modules/video_chroma/i420_nv12.c | 95 ++++++++++++++++++++++++++++++++++++++++
 2 files changed, 100 insertions(+)
 create mode 100644 modules/video_chroma/i420_nv12.c

diff --git a/modules/video_chroma/Modules.am b/modules/video_chroma/Modules.am
index f7195f9..0138826 100644
--- a/modules/video_chroma/Modules.am
+++ b/modules/video_chroma/Modules.am
@@ -20,6 +20,10 @@ SOURCES_i422_i420 = \
 	i422_i420.c \
 	$(NULL)

+SOURCES_i420_nv12 = \
+	i420_nv12.c \
+	$(NULL)
+
 SOURCES_grey_yuv = \
 	grey_yuv.c \
 	$(NULL)
@@ -37,6 +41,7 @@ SOURCES_rv32 = rv32.c
 libvlc_LTLIBRARIES += \
 	libi420_rgb_plugin.la \
 	libi420_yuy2_plugin.la \
+	libi420_nv12_plugin.la \
 	libi422_i420_plugin.la \
 	libi422_yuy2_plugin.la \
 	libgrey_yuv_plugin.la \
diff --git a/modules/video_chroma/i420_nv12.c b/modules/video_chroma/i420_nv12.c
new file mode 100644
index 0000000..702468b
--- /dev/null
+++ b/modules/video_chroma/i420_nv12.c
@@ -0,0 +1,95 @@
+/*****************************************************************************
+ * i420_nv12.c : YUV to NV12 conversion module for vlc
+ *****************************************************************************
+ * Copyright (C) 2013 VideoLabs
+ * $Id$
+ *
+ * Authors: Julien 'Lta' BALLET <contact@lta.io>
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+/*****************************************************************************
+ * Preamble
+ *****************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include <vlc_plugin.h>
+#include <vlc_filter.h>
+#include <vlc_cpu.h>
+
+/*****************************************************************************
+ * Local and extern prototypes.
+ *****************************************************************************/
+static int  Activate               (vlc_object_t *);
+
+static picture_t *I420_NV12_Filter (filter_t *, picture_t *);
+static void I420_NV12              (filter_t *, picture_t *, picture_t *);
+
+vlc_module_begin ()
+    set_description( N_("Conversions from I420 to NV12") )
+    set_capability( "video filter2", 100 )
+    set_callbacks( Activate, NULL )
+vlc_module_end ()
+
+/*****************************************************************************
+ * Activate: allocate a chroma function
+ *****************************************************************************
+ * This function allocates and initializes a chroma function
+ *****************************************************************************/
+static int Activate(vlc_object_t *this)
+{
+    filter_t *filter = (filter_t *)this;
+
+    if (filter->fmt_in.video.i_width & 1
+        || filter->fmt_in.video.i_height & 1)
+        return VLC_EGENERIC;
+
+    if (filter->fmt_in.video.i_width != filter->fmt_out.video.i_width
+        || filter->fmt_in.video.i_height != filter->fmt_out.video.i_height)
+        return VLC_EGENERIC;
+
+    if (filter->fmt_in.video.i_chroma != VLC_CODEC_I420
+        && filter->fmt_out.video.i_chroma != VLC_CODEC_NV12)
+        return VLC_EGENERIC;
+
+    filter->pf_video_filter = I420_NV12_Filter;
+    return VLC_SUCCESS;
+}
+
+static void I420_NV12  (filter_t *filter, picture_t *src, picture_t *dst)
+{
+    VLC_UNUSED(filter);
+    uint8_t u, v;
+
+    // Copy Y plane unchanged
+    memcpy(dst->p[0].p_pixels, src->p[0].p_pixels, src->p[0].i_lines * src->p[0].i_pitch);
+
+    // Interleave UV planes
+    for(int i = 0; i < src->p[1].i_lines; i++) {
+        for(int j = 0; j < src->p[1].i_pitch; j++) {
+            u = src->p[1].p_pixels[i * src->p[1].i_pitch + j];
+            v = src->p[2].p_pixels[i * src->p[1].i_pitch + j];
+            dst->p[1].p_pixels[i * dst->p[1].i_pitch + 2 * j] = u;
+            dst->p[1].p_pixels[i * dst->p[1].i_pitch + 2 * j + 1] = v;
+        }
+    }
+}
+
+VIDEO_FILTER_WRAPPER( I420_NV12 )
--
1.8.1.2

